//
// Licensed to the Apache Software Foundation (ASF) under one or more
// contributor license agreements.  See the NOTICE file distributed with
// this work for additional information regarding copyright ownership.
// The ASF licenses this file to You under the Apache License, Version 2.0
// (the "License"); you may not use this file except in compliance with
// the License.  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

//  .Net StockTrader Sample WCF Application for Benchmarking, Performance Analysis and Design Considerations for Service-Oriented Applications

//======================================================================================================
// This is one of two optional host programs for the Order Processor Service .  In this case a Console  
// application.  You can also optionally run the Windows host.  
//======================================================================================================
//======================================================================================================
// Code originally contributed by Microsoft Corporation.
// This contribution to the Stonehenge project is limited strictly 
// to the source code that is submitted in this submission.  
// Any technology, including underlying platform technology, 
// that is referenced or required by the submitted source code 
// is not a part of the contribution.  
// For example and not by way of limitation, 
// any systems/Windows libraries (WPF, WCF, ASP.NET etc.) 
// required to run the submitted source code is not a part of the contribution
//======================================================================================================

using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Threading;
using System.Net;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Configuration;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Activation;
using Trade.OrderProcessorServiceConfigurationSettings;
using Trade.OrderProcessorContract;
using Trade.OrderProcessorImplementation;
using Trade.Utility;


namespace Trade.OrderProcessorConsoleHost
{
    class OrderProcessor_ConsoleHost
    {
        //private static EndPointBehaviors endpointBehaviorList;
        //public static List<ServiceHostInfo> startupList;

        /// <summary>p
        /// The program entry class. Note how this simply inherits from the provided base class.
        /// </summary>
        class MyHost 
        {
           
            public void startUp()
            {
                string shortHostName = System.Net.Dns.GetHostName();
                IPHostEntry myEntry = System.Net.Dns.GetHostEntry(shortHostName);
                string myName = myEntry.HostName;
                //The key call to create our list of runtime hosts to be initialized.
                //List<ServiceHostInfo> startupList = new List<ServiceHostInfo>(new ServiceHostInfo[] { new ServiceHostInfo(Settings.SELFHOST_VHOST_NAME, false, null, new object[] { new Trade.BusinessServiceImplementation.ErrorBehaviorAttribute() }, new TradeServiceWcf()) });
                ////Stock call to startup the Master Host.
                //base.startService(new Settings(), new ConfigurationService(), new NodeCommunication(), null, new ConfigurationActions(true), startupList, null, new object[] { typeof(IOrderProcessor) });

                Settings thisSettings = new Settings();

                Uri TradeServiceUriBase = new Uri("http://" + myName + ":8000/tradeorderprocessor");

                //Create a ServiceHost for the Orderprocessor service.
                Type serviceType = typeof(OrderProcessor);
                ServiceHost host = new ServiceHost(serviceType, TradeServiceUriBase);
                addEndpointsAndBehaviors(host, TradeServiceUriBase);
                host.Open();

            }

            private ServiceHost addEndpointsAndBehaviors(ServiceHost host, Uri uri)
            {
                ServiceMetadataBehavior mexbehavior = new ServiceMetadataBehavior();
                mexbehavior.HttpGetEnabled = true;
                mexbehavior.HttpGetUrl = uri;
                host.Description.Behaviors.Add(mexbehavior);
                Binding mexHttpBinding = MetadataExchangeBindings.CreateMexHttpBinding();
                ServiceEndpoint sepMex = host.AddServiceEndpoint(typeof(IMetadataExchange), mexHttpBinding, uri.AbsoluteUri + "/mex");
                BasicHttpBinding httpBinding = new BasicHttpBinding("OpsHost_BasicHttpBinding");
                WSHttpBinding wsHttpBinding = new WSHttpBinding("OpsHost_WsHttpBinding_MSec");
                CustomBinding customBinding = new CustomBinding("Host_CustomHttpBinding_M_Security_OPS");
                host.AddServiceEndpoint(typeof(IOrderProcessor), httpBinding, "");
                host.AddServiceEndpoint(typeof(IOrderProcessor), wsHttpBinding, "msec");
				host.AddServiceEndpoint(typeof(IOrderProcessor), customBinding, "sec");

                //Describe for Console output
                StockTraderUtility.DescribeService(host); 
                
                return host;
            }
        }

        [STAThread]
        static void Main(string[] args)
        {
            Console.SetWindowSize(Console.LargestWindowWidth - 30, Console.LargestWindowHeight - 30);
            Console.Title = ".NET StockTrader Order Processor Service Host";
            MyHost myHost = new MyHost();
            myHost.startUp();
            
            Console.WriteLine(" {0} is started.", Console.Title);
            Console.WriteLine();
            Console.WriteLine("Press Enter to quit.");
            Console.ReadLine();
        }
    }
}
