//
// Licensed to the Apache Software Foundation (ASF) under one or more
// contributor license agreements.  See the NOTICE file distributed with
// this work for additional information regarding copyright ownership.
// The ASF licenses this file to You under the Apache License, Version 2.0
// (the "License"); you may not use this file except in compliance with
// the License.  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

//  .NET StockTrader Sample WCF Application for Benchmarking, Performance Analysis and Design Considerations for Service-Oriented Applications

using System;
using System.Collections.Generic;
using System.Text;
using Trade.StockTraderWebApplicationSettings;

namespace Trade.StockTraderWebApplicationModelClasses
{
    /// <summary>
    /// Model class for displaying special/augmented order data in a web page.
    /// </summary>
    public sealed class TotalOrdersUI
    {
        private List<OrderDataUI> _orders;
        private decimal _subtotalSell;
        private decimal _subtotalBuy;
        private decimal _txnFeesSubtotal;
        
        public TotalOrdersUI()
        {

        }

        public TotalOrdersUI(List<OrderDataUI>orders, decimal subtotalSell,decimal subtotalBuy, decimal txnFeesSubtotal)
        {
            _orders = orders;
            _subtotalSell = subtotalSell;
            _subtotalBuy = subtotalBuy;
            _txnFeesSubtotal = txnFeesSubtotal;            
        }

        public List<OrderDataUI> orders
        {
            get
            {
                return _orders;
            }
        }

        public decimal subtotalSell
        {
            get
            {
                return _subtotalSell;
            }
        }

        public decimal subtotalBuy
        {
            get
            {
                return _subtotalBuy;
            }
        }

        public decimal txnFeesSubtotal
        {
            get
            {
                return _txnFeesSubtotal;
            }
        }

        public string netImpactCashBalance
        {
            get
            {
                decimal netImpact = (this._subtotalSell - this._subtotalBuy - this._txnFeesSubtotal);
                return DataFormatHelper.NumberWithStyledArrow(netImpact);
            }
        }
    }
}