//
// Licensed to the Apache Software Foundation (ASF) under one or more
// contributor license agreements.  See the NOTICE file distributed with
// this work for additional information regarding copyright ownership.
// The ASF licenses this file to You under the Apache License, Version 2.0
// (the "License"); you may not use this file except in compliance with
// the License.  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

using System;

    /// <summary>
    /// This class trims input characters from a string, based on specified length.  Helps to prevent
    /// buffer overrun attacks.  Could be augmented with strip-out of script-injection attack characters, 
    /// although ASP.NET automatically does this for common special characters.
    /// </summary>
    public static class Input
    {
        /// <summary>
        /// This method trims input characters from a string, based on specified length.</summary>
        /// <param name="inputString">String to clean.</param> 
        /// <param name="maxLength">Cut off length to return.</param> 
        public static string InputText(string inputString, int maxLength)
        {
            // check incoming parameters for null or blank string
            if ((inputString != null) && (inputString != String.Empty))
            {
                inputString = inputString.Trim();

                //chop the string incase the client-side max length
                //fields are bypassed to prevent buffer over-runs
                if (inputString.Length > maxLength)
                    inputString = inputString.Substring(0, maxLength);
            }
            return inputString;
        }
    }
