/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.stonehenge.stocktrader.services;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.axiom.soap.SOAP12Constants; 
import org.apache.axis2.AxisFault;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.neethi.Policy;
import org.apache.neethi.PolicyEngine;
import org.apache.stonehenge.stocktrader.CustomOrderBean;
import org.apache.stonehenge.stocktrader.util.StockTraderUtility;

import traderorderhost.trade.OrderProcessorStub;
import traderorderhost.trade.SubmitOrder;
import traderorderhost.trade.SubmitOrderTransactedQueue;

public class TradeOrderServiceClient extends ServiceClient {

	private static TradeOrderServiceClient self = null;
	private Policy securityPolicy = null;

	private TradeOrderServiceClient() throws IOException {
		super();
		// CHECKME .. there might be a better of doing this
		ClassLoader cl = MessageContext.getCurrentMessageContext().getAxisService().getClassLoader();
		InputStream is = cl.getResourceAsStream(StockTraderUtility.TRADE_ORDER_SERVICE_SECURITY_FILE);
		securityPolicy = PolicyEngine.getPolicy(is);
	}

	public static final TradeOrderServiceClient getInstance()
			throws IOException {
		if (self == null) {
			self = new TradeOrderServiceClient();
		}
		return self;
	}

	public void SubmitOrderTrasactedQueue(CustomOrderBean order) throws IOException {
		TradeOrderConfig config = geTradeOrderConfig();
		OrderProcessorStub stub = new OrderProcessorStub(config.getEndpointURL());
		ServiceClient serviceClient = stub._getServiceClient();
		serviceClient.getOptions().setProperty(HTTPConstants.CHUNKED,  Boolean.FALSE);
		if (config.isSecure()) {
			enableSecurity(stub);
			serviceClient.getOptions().setSoapVersionURI(SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI); 
		}
		
		SubmitOrder param = new SubmitOrder();
		param.setOrder(order);
		stub.submitOrder(param);
	}

	private TradeOrderConfig geTradeOrderConfig() throws IOException {
		TradeConfigServiceClient client = TradeConfigServiceClient.getInstance();
		return client.getTradeOrderConfig();
	}

	private void enableSecurity(OrderProcessorStub stub) throws AxisFault {
		ServiceClient serviceClient = stub._getServiceClient();
		AxisService axisService = serviceClient.getAxisService();
		axisService.getPolicySubject().attachPolicy(securityPolicy);
		serviceClient.engageModule("rampart");
	}
}
