/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package org.apache.stonehenge.stocktrader.services;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.stonehenge.stocktrader.CustomAccountBean;
import org.apache.stonehenge.stocktrader.CustomAccountProfileBean;
import org.apache.stonehenge.stocktrader.CustomHoldingBean;
import org.apache.stonehenge.stocktrader.CustomMarketSummaryBean;
import org.apache.stonehenge.stocktrader.CustomOrderBean;
import org.apache.stonehenge.stocktrader.CustomQuoteBean;
import org.apache.stonehenge.stocktrader.dal.DAOException;
import org.tempuri.TradeServiceWcfSkeleton;

import com.ibm.websphere.samples.trade.ArrayOfHoldingDataBean;
import com.ibm.websphere.samples.trade.ArrayOfOrderDataBean;
import com.ibm.websphere.samples.trade.Buy;
import com.ibm.websphere.samples.trade.BuyResponse;
import com.ibm.websphere.samples.trade.GetAccountData;
import com.ibm.websphere.samples.trade.GetAccountDataResponse;
import com.ibm.websphere.samples.trade.GetAccountProfileData;
import com.ibm.websphere.samples.trade.GetAccountProfileDataResponse;
import com.ibm.websphere.samples.trade.GetClosedOrders;
import com.ibm.websphere.samples.trade.GetClosedOrdersResponse;
import com.ibm.websphere.samples.trade.GetHolding;
import com.ibm.websphere.samples.trade.GetHoldingResponse;
import com.ibm.websphere.samples.trade.GetHoldings;
import com.ibm.websphere.samples.trade.GetHoldingsResponse;
import com.ibm.websphere.samples.trade.GetMarketSummary;
import com.ibm.websphere.samples.trade.GetMarketSummaryResponse;
import com.ibm.websphere.samples.trade.GetOrders;
import com.ibm.websphere.samples.trade.GetOrdersResponse;
import com.ibm.websphere.samples.trade.GetQuote;
import com.ibm.websphere.samples.trade.GetQuoteResponse;
import com.ibm.websphere.samples.trade.GetTopOrders;
import com.ibm.websphere.samples.trade.GetTopOrdersResponse;
import com.ibm.websphere.samples.trade.HoldingDataBean;
import com.ibm.websphere.samples.trade.IsOnline;
import com.ibm.websphere.samples.trade.Login;
import com.ibm.websphere.samples.trade.LoginResponse;
import com.ibm.websphere.samples.trade.Logout;
import com.ibm.websphere.samples.trade.LogoutResponse;
import com.ibm.websphere.samples.trade.OrderDataBean;
import com.ibm.websphere.samples.trade.Register;
import com.ibm.websphere.samples.trade.RegisterResponse;
import com.ibm.websphere.samples.trade.Sell;
import com.ibm.websphere.samples.trade.SellEnhanced;
import com.ibm.websphere.samples.trade.SellEnhancedResponse;
import com.ibm.websphere.samples.trade.SellResponse;
import com.ibm.websphere.samples.trade.UpdateAccountProfile;
import com.ibm.websphere.samples.trade.UpdateAccountProfileResponse;

public class TradeServiceImpl extends TradeServiceWcfSkeleton {
	private static final Log logger = LogFactory.getLog(TradeServiceImpl.class);

	private final TraderServiceManager mgr = new TraderServiceManager();

	public BuyResponse buy(Buy buy) {
		CustomOrderBean order;
		try {
			order = mgr.buy(buy.getUserID(), buy.getSymbol(), buy.getQuantity());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		BuyResponse buyResponse = new BuyResponse();
		buyResponse.setBuyReturn(order);
		return buyResponse;
	}

	public GetAccountDataResponse getAccountData(GetAccountData getAccountData) {
		CustomAccountBean accountData;
		try {
			accountData = mgr.getAccountData(getAccountData.getUserID());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetAccountDataResponse getAccountDataResponse = new GetAccountDataResponse();
		getAccountDataResponse.setGetAccountDataReturn(accountData);
		return getAccountDataResponse;
	}

	public GetAccountProfileDataResponse getAccountProfileData(GetAccountProfileData getAccountProfileData) {
		CustomAccountProfileBean accountProfileData;
		try {
			accountProfileData = mgr.getAccountProfileData(getAccountProfileData.getUserID());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetAccountProfileDataResponse response = new GetAccountProfileDataResponse();
		response.setGetAccountProfileDataReturn(accountProfileData);
		return response;
	}

	public GetClosedOrdersResponse getClosedOrders(GetClosedOrders getClosedOrders) {
		List<CustomOrderBean> closedOrders;
		try {
			closedOrders = mgr.getClosedOrders(getClosedOrders.getUserID());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetClosedOrdersResponse response = new GetClosedOrdersResponse();
		ArrayOfOrderDataBean arrayOfOrderBean = new ArrayOfOrderDataBean();
		arrayOfOrderBean.setOrderDataBean(toArray(closedOrders));
		response.setGetClosedOrdersReturn(arrayOfOrderBean);
		return response;
	}

	public GetHoldingResponse getHolding(GetHolding getHolding) {
		CustomHoldingBean holding;
		try {
			holding = mgr.getHolding(getHolding.getUserID(), getHolding.getHoldingID());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetHoldingResponse response = new GetHoldingResponse();
		response.setGetHoldingReturn(holding);
		return response;
	}

	public GetHoldingsResponse getHoldings(GetHoldings getHoldings) {
		List<CustomHoldingBean> holdingBeans;
		try {
			holdingBeans = mgr.getHoldings(getHoldings.getUserID());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetHoldingsResponse response = new GetHoldingsResponse();
		ArrayOfHoldingDataBean param = new ArrayOfHoldingDataBean();
		param.setHoldingDataBean(toHoldingsArray(holdingBeans));
		response.setGetHoldingsReturn(param);
		return response;
	}

	public GetMarketSummaryResponse getMarketSummary(GetMarketSummary getMarketSummary) {
		CustomMarketSummaryBean param;
		try {
			param = mgr.getMarketSummary();
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetMarketSummaryResponse response = new GetMarketSummaryResponse();
		response.setGetMarketSummaryReturn(param);
		return response;
	}

	public GetOrdersResponse getOrders(GetOrders getOrders) {
		List<CustomOrderBean> orders;
		try {
			orders = mgr.getOrders(getOrders.getUserID());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetOrdersResponse response = new GetOrdersResponse();
		ArrayOfOrderDataBean param = new ArrayOfOrderDataBean();
		param.setOrderDataBean(toArray(orders));
		response.setGetOrdersReturn(param);
		return response;
	}

	public GetQuoteResponse getQuote(GetQuote getQuote) {
		CustomQuoteBean param;
		try {
			param = mgr.getQuote(getQuote.getSymbol());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetQuoteResponse response = new GetQuoteResponse();
		response.setGetQuoteReturn(param);
		return response;
	}

	public GetTopOrdersResponse getTopOrders(GetTopOrders getTopOrders) {
		List<CustomOrderBean> topOrders;
		try {
			topOrders = mgr.getTopOrders(getTopOrders.getUserID());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		GetTopOrdersResponse response = new GetTopOrdersResponse();
		ArrayOfOrderDataBean param = new ArrayOfOrderDataBean();
		param.setOrderDataBean(toArray(topOrders));
		response.setGetTopOrdersReturn(param);
		return response;
	}

	public void isOnline(IsOnline isOnline) {
		if (logger.isDebugEnabled()) {
			logger.debug("TradeServiceImpl.isOnline()");
		}
	}

	public LoginResponse login(Login login) {
		CustomAccountBean param;
		try {
			param = mgr.login(login.getUserID(), login.getPassword());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		LoginResponse response = new LoginResponse();
		response.setLoginReturn(param);
		return response;
	}

	public LogoutResponse logout(Logout logout) {
		try {
			mgr.logout(logout.getUserID());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		LogoutResponse response = new LogoutResponse();
		return response;
	}

	public RegisterResponse register(Register register) {
		CustomAccountBean param;
		try {
			param = mgr.register(
                    register.getUserID(),
                    register.getPassword(),
					register.getFullname(),
                    register.getAddress(),
                    register.getEmail(),
                    register.getCreditcard(),
                    register.getOpenBalance());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		RegisterResponse response = new RegisterResponse();
		response.setRegisterReturn(param);
		return response;
	}

	public SellResponse sell(Sell sell) {
		CustomOrderBean param;
		try {
			param = mgr.sell(sell.getUserID(), sell.getHoldingID(), sell.getOrderProcessingMode());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		SellResponse response = new SellResponse();
		response.setSellReturn(param);
		return response;
	}

	public SellEnhancedResponse sellEnhanced(SellEnhanced sellEnhanced) {
		CustomOrderBean param;
		try {
			param = mgr.sellEnhanced(sellEnhanced.getUserID(), sellEnhanced.getHoldingID(), sellEnhanced.getQuantity());
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}
		SellEnhancedResponse response = new SellEnhancedResponse();
		response.setSellEnhancedReturn(param);
		return response;
	}

	public UpdateAccountProfileResponse updateAccountProfile(UpdateAccountProfile updateAccountProfile) {
		CustomAccountProfileBean accountProfile = new CustomAccountProfileBean(updateAccountProfile.getProfileData());
		CustomAccountProfileBean param;
		try {
			param = mgr.updateAccountProfile(accountProfile);
		} catch (DAOException e) {
			logger.debug("", e);
			throw new RuntimeException(e);
		}

		UpdateAccountProfileResponse response = new UpdateAccountProfileResponse();
		response.setUpdateAccountProfileReturn(param);
		return response;
	}

	private OrderDataBean[] toArray(List<CustomOrderBean> orders) {
		OrderDataBean[] arrayOfOrders = new OrderDataBean[orders.size()];
		return (OrderDataBean[]) orders.toArray(arrayOfOrders);
	}

	private HoldingDataBean[] toHoldingsArray(List<CustomHoldingBean> holdings) {
		HoldingDataBean[] arrayOfHoldings = new HoldingDataBean[holdings.size()];
		return (HoldingDataBean[]) holdings.toArray(arrayOfHoldings);
	}
}
