/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.stonehenge.stocktrader;

import java.math.BigDecimal;
import java.util.Calendar;

import com.ibm.websphere.samples.trade.HoldingDataBean;

/**
 * This class is to be used as the data model for holding information.
 */
public class CustomHoldingBean extends HoldingDataBean {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3294871357941879094L;
	private int accountID;

	public CustomHoldingBean() {
	}

	/**
	 * Constructs a data model instance with holding information.
	 * 
	 * @param accountId
	 *            the account ID of the buyer
	 * @param holdingID
	 *            the holding ID of the stock
	 * @param quantity
	 *            the quantity of the stock
	 * @param purchasePrice
	 *            the purchase price of the stock
	 * @param purchaseDate
	 *            the date of the purchase of the stock
	 * @param quoteID
	 *            the quote ID of the stock
	 */
	public CustomHoldingBean(int accountId, int holdingID, double quantity,
			BigDecimal purchasePrice, Calendar purchaseDate, String quoteID) {
		setAccountID(accountId);
		setHoldingID(holdingID);
		setQuantity(quantity);
		setPurchasePrice(purchasePrice);
		setPurchaseDate(purchaseDate);
		setQuoteID(quoteID);
	}

	public CustomHoldingBean(int holdingID, double quantity,
			BigDecimal purchasePrice, Calendar purchaseDate, String quoteID,
			int accountID) {
		setHoldingID(holdingID);
		setAccountID(accountID);
		setQuantity(quantity);
		setPurchasePrice(purchasePrice);
		setPurchaseDate(purchaseDate);
		setQuoteID(quoteID);
	}

	public int getAccountID() {
		return accountID;
	}

	public void setAccountID(int accountID) {
		this.accountID = accountID;
	}
}
