/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.stonehenge.stocktrader;

import java.math.BigDecimal;

/**
 * This class to be used as the data model for quote information.
 */
public class QuoteBean {

	private String symbol;
	private String companyName;
	private double volume;
	private BigDecimal price;
	private BigDecimal open;
	private BigDecimal low;
	private BigDecimal high;
	private double change;

	public QuoteBean() {
	}

	/**
	 * Constructs an instance for quote information.
	 * 
	 * @param symbol
	 * @param companyName
	 * @param volume
	 * @param price
	 * @param open
	 * @param low
	 * @param high
	 * @param change
	 */
	public QuoteBean(String symbol, String companyName, double volume,
			BigDecimal price, BigDecimal open, BigDecimal low, BigDecimal high,
			double change) {
		this.symbol = symbol;
		this.companyName = companyName;
		this.volume = volume;
		this.price = price;
		this.open = open;
		this.low = low;
		this.high = high;
		this.change = change;
	}

	public String getSymbol() {
		return symbol;
	}

	public void setSymbol(String symbol) {
		this.symbol = symbol;
	}

	public String getCompanyName() {
		return companyName;
	}

	public void setCompanyName(String companyName) {
		this.companyName = companyName;
	}

	public double getVolume() {
		return volume;
	}

	public void setVolume(double volume) {
		this.volume = volume;
	}

	public BigDecimal getPrice() {
		return price;
	}

	public void setPrice(BigDecimal price) {
		this.price = price;
	}

	public BigDecimal getOpen() {
		return open;
	}

	public void setOpen(BigDecimal open) {
		this.open = open;
	}

	public BigDecimal getLow() {
		return low;
	}

	public void setLow(BigDecimal low) {
		this.low = low;
	}

	public BigDecimal getHigh() {
		return high;
	}

	public void setHigh(BigDecimal high) {
		this.high = high;
	}

	public double getChange() {
		return change;
	}

	public void setChange(double change) {
		this.change = change;
	}
}
