/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package org.apache.stonehenge.stocktrader.dal;

import java.math.BigDecimal;
import java.util.List;

import org.apache.stonehenge.stocktrader.CustomAccountBean;
import org.apache.stonehenge.stocktrader.CustomAccountProfileBean;
import org.apache.stonehenge.stocktrader.CustomHoldingBean;
import org.apache.stonehenge.stocktrader.CustomOrderBean;

public interface CustomerDAO {

	public CustomHoldingBean getHoldingForUpdate(int orderId)
			throws DAOException;

	public CustomHoldingBean getHolding(String userId, int holdingID)
			throws DAOException;

	public void updateAccountBalance(int accountId, BigDecimal total)
			throws DAOException;

	public CustomAccountBean login(String userId, String password)
			throws DAOException;

	public void logoutUser(String userId) throws DAOException;

	public CustomAccountBean getCustomerByUserId(String userId)
			throws DAOException;

	public CustomAccountProfileBean getAccountProfileData(String userId)
			throws DAOException;

	public List<CustomOrderBean> getOrders(String userId, boolean top,
			int maxTop, int maxDefault) throws DAOException;

	public List<CustomOrderBean> getClosedOrders(String userId)
			throws DAOException;

	public void insertAccountProfile(CustomAccountProfileBean accountProfileBean)
			throws DAOException;

	public void insertAccount(CustomAccountBean accountBean)
			throws DAOException;

	public CustomAccountProfileBean update(
			CustomAccountProfileBean customAccountProfile) throws DAOException;

	public List<CustomHoldingBean> getHoldings(String userID)
			throws DAOException;

}
